import logging
import functools

from typing import NamedTuple

from .exceptions import UnhandledRecordData

logger = logging.getLogger(__name__)


# This is needed so that mypy is happy with our return annotations
class Record:  # type: ignore
    pass


def _generate_records():
    def _inner(func):
        @functools.wraps(func)
        def _fn(*args, **kwargs):
            headers, rows = func(*args, **kwargs)

            if not headers:
                logger.error("no headers were returned by the query...")
                logger.error(f"{headers=}")
                raise UnhandledRecordData("No headers were returned by the SQL query.")

            if not isinstance(headers, tuple):
                logger.error("the query result headers were not returned as a tuple...")
                logger.error(f"{headers=}")
                raise UnhandledRecordData("The query result headers were not returned as a tuple.")

            if not rows:
                logger.info("the query returned no results...")
                return []

            if not isinstance(rows, list):
                logger.error("the query result rows were not returned as a list...")
                logger.error(f"{rows=}")
                raise UnhandledRecordData("The query results were not returned as a list.")

            # headers variable contains the values listed here:
            # https://github.com/mkleehammer/pyodbc/wiki/Cursor#description
            annotations = [(header[0].replace(" ", "_") or "missing_column_name", header[1]) for header in headers]
            Record = NamedTuple("Record", annotations)  # type: ignore
            return [Record(*row) for row in rows]  # type: ignore # pylint: disable=not-callable

        return _fn

    return _inner
