from typeguard import typechecked
from . import _client
from .exceptions import InvalidSQLQuery, ParameterError


class SantisedWriteQuery(str):
    def __new__(cls, string: str, original_value: str):
        cls.original_value = original_value
        # If the following line is not ignored, you get this error:
        #
        # error: Too many arguments for "__new__" of "object"
        #
        # which looks like the str equivalent of this bug: https://github.com/python/typeshed/issues/2686
        return super().__new__(cls, string)  # type: ignore


@typechecked
def _sanitise(query: str):
    original_query = query
    query = query.lower().strip()

    query = _client._remove_comments(query)

    if not query.startswith("insert") and not query.startswith("update") and not query.startswith("delete"):
        raise InvalidSQLQuery(
            "Invalid Write Operation. SQL Query must start with one of the following: insert, update, delete"
        )

    injection_message = (
        "An unexpected statement delimiter (;) was found in the SQL Query, this could be an SQL Injection attempt."
    )
    _client._check_for_sql_injection(query, injection_message)
    if query.count(";") == 0:
        query = "".join([query, ";"])
    return SantisedWriteQuery(query, original_query)


def _query(connection, *, query: str, values: tuple = ()):
    if not isinstance(query, SantisedWriteQuery):
        raise ParameterError("The SQL Query provided must be santised first via the sanitise_query function.")
    cursor = _client._execute(connection, query=query, values=values)
    cursor.commit()
