from typing import TypedDict, Union, Optional


class MSSQLServer(TypedDict):
    r"""
- Contains remote SQL Server connection details for a Microsoft SQL Server

- It is used as an _input_ to many of the actions in this plugin

- The example below shows the dictionary structure in YAML

- See the ___required___ and _optional_ keys underneath the example

!!! warning "Supplying Credentials"

    You should ___always___ use a PPA Vault integration to provide credentials to a plugin action.

!!! example

    ```yaml linenums="1"
    mssql:
      address: sql-server.internal.net
      instance: sqlserver
      database: master
      username: example.username
      password: example.password
      port: 1433
    ```

!!! danger "Required Keys"

    __address__: The target IP or DNS address.

    __database__: The database to connect to.

    __username__: Username for authentication.

    __password__: Password for authentication.

!!! info "Optional Keys"

    __instance__: The SQL instance being connected to (defaults to MSSQLSERVER).

    __port__: The port to use when connecting (defaults to _1433_).

    """
    address: str
    database: str
    username: str
    password: str
    instance: Optional[str] = "MSSQLSERVER"
    port: Optional[Union[int, str]] = 1433
